library amp;

(* This is the Delphi port of the amp plugin that comes with the original
   LADSPA sdk. The code is straight from the original with very little changes.

                                                Zarko Asenov 2005

(******************************************************************************)

(* amp.c

   Free software by Richard W.E. Furse. Do with as you will. No
   warranty.

   This LADSPA plugin provides simple mono and stereo amplifiers.

   This file has poor memory protection. Failures during malloc() will
   not recover nicely.  *)

(*****************************************************************************)

uses
  ladspa;


(*****************************************************************************)

(* The port numbers for the plugin: *)

const
     AMP_CONTROL = 0;
     AMP_INPUT1  = 1;
     AMP_OUTPUT1 = 2;
     AMP_INPUT2  = 3;
     AMP_OUTPUT2 = 4;

(*****************************************************************************)

(* The structure used to hold port connection information and state
   (actually gain controls require no further state). *)

type
    PAmplifier = ^TAmplifier;
    TAmplifier = record

  (* Ports:
     ------ *)

      m_pfControlValue: PLADSPA_Data;
      m_pfInputBuffer1: PLADSPA_Data;
      m_pfOutputBuffer1: PLADSPA_Data;
      m_pfInputBuffer2: PLADSPA_Data;  (* (Not used for mono) *)
      m_pfOutputBuffer2: PLADSPA_Data; (* (Not used for mono) *)
    end;

(*****************************************************************************)

(* Construct a new plugin instance. *)
function instantiateAmplifier(Descriptor: PLadspa_Descriptor; SampleRate: Cardinal): TLADSPA_Handle; cdecl;
begin
  GetMem(Result, SizeOf(TAmplifier));
end;


(*****************************************************************************)

(* Connect a port to a data location. *)
procedure connectPortToAmplifier(Instance: TLADSPA_Handle; Port: Cardinal; DataLocation: PLADSPA_Data); cdecl;
var
  psAmplifier: PAmplifier;
begin
  psAmplifier := PAmplifier(Instance);

  case Port of
  AMP_CONTROL:
    psAmplifier.m_pfControlValue := DataLocation;
  AMP_INPUT1:
    psAmplifier.m_pfInputBuffer1 := DataLocation;
  AMP_OUTPUT1:
    psAmplifier.m_pfOutputBuffer1 := DataLocation;
  AMP_INPUT2:
    (* (This should only happen for stereo.) *)
    psAmplifier.m_pfInputBuffer2 := DataLocation;
  AMP_OUTPUT2:
    (* (This should only happen for stereo.) *)
    psAmplifier.m_pfOutputBuffer2 := DataLocation;
  end;
end;

(*****************************************************************************)

procedure runMonoAmplifier(Instance: TLADSPA_Handle; SampleCount: Cardinal); cdecl;
var
  pfInput: PLADSPA_Data;
  pfOutput: PLADSPA_Data;
  fGain: TLADSPA_Data;
  psAmplifier: PAmplifier;
  lSampleIndex: Cardinal;
begin
  psAmplifier := PAmplifier(Instance);

  pfInput := psAmplifier.m_pfInputBuffer1;
  pfOutput := psAmplifier.m_pfOutputBuffer1;
  fGain := psAmplifier.m_pfControlValue^;

  for lSampleIndex := 0 to SampleCount - 1 do
  begin
    pfOutput^ := pfInput^ * fGain;
    inc(pfOutput);
    inc(pfInput);
  end;
end;

(*****************************************************************************)

procedure runStereoAmplifier(Instance: TLADSPA_Handle; SampleCount: Cardinal); cdecl;
var
  pfInput: PLADSPA_Data;
  pfOutput: PLADSPA_Data;
  fGain: TLADSPA_Data;
  psAmplifier: PAmplifier;
  lSampleIndex: Cardinal;
begin

  psAmplifier := PAmplifier(Instance);

  fGain := psAmplifier.m_pfControlValue^;

  pfInput := psAmplifier.m_pfInputBuffer1;
  pfOutput := psAmplifier.m_pfOutputBuffer1;
  for lSampleIndex := 0 to SampleCount - 1 do
  begin
    pfOutput^ := pfInput^ * fGain;
    inc(pfOutput);
    inc(pfInput);
  end;

  pfInput := psAmplifier.m_pfInputBuffer2;
  pfOutput := psAmplifier.m_pfOutputBuffer2;
  for lSampleIndex := 0 to SampleCount - 1 do
  begin
    pfOutput^ := pfInput^ * fGain;
    inc(pfOutput);
    inc(pfInput);
  end;
end;

(*****************************************************************************)

(* Throw away a simple amplifier. *)
procedure cleanupAmplifier(Instance: TLADSPA_Handle); cdecl;
begin
  FreeMem(Instance);
end;

(*****************************************************************************)

var
   g_psMonoDescriptor: PLADSPA_Descriptor = NIL;
   g_psStereoDescriptor: PLADSPA_Descriptor = NIL;

(*****************************************************************************)

(* _init() is called automatically when the plugin library is first
   loaded. *)

procedure _init(); cdecl;
var
  pcPortNames: PPChar;
  piPortDescriptors: PLADSPA_PortDescriptor;
  psPortRangeHints: PLADSPA_PortRangeHint;
begin
  GetMem(g_psMonoDescriptor, SizeOf(TLADSPA_Descriptor));
  GetMem(g_psStereoDescriptor, SizeOf(TLADSPA_Descriptor));

  if g_psMonoDescriptor <> NIL then
  begin
    g_psMonoDescriptor.UniqueID := 1048;
    g_psMonoDescriptor.lLabel := 'amp_mono';
    g_psMonoDescriptor.Properties := LADSPA_PROPERTY_HARD_RT_CAPABLE;
    g_psMonoDescriptor.Name := 'Mono Amplifier';
    g_psMonoDescriptor.Maker := 'Richard Furse (LADSPA example plugins) Kylix port by Jarko Asenov';
    g_psMonoDescriptor.Copyright := 'None';
    g_psMonoDescriptor.PortCount := 3;

    GetMem(piPortDescriptors, 3 * SizeOf(TLADSPA_PortDescriptor));
    g_psMonoDescriptor.PortDescriptors := piPortDescriptors;
    PLADSPA_PortDescriptor(Integer(piPortDescriptors) + AMP_CONTROL * SizeOf(TLADSPA_PortDescriptor))^ := LADSPA_PORT_INPUT or LADSPA_PORT_CONTROL;
    PLADSPA_PortDescriptor(Integer(piPortDescriptors) + AMP_INPUT1 * SizeOf(TLADSPA_PortDescriptor))^ := LADSPA_PORT_INPUT or LADSPA_PORT_AUDIO;
    PLADSPA_PortDescriptor(Integer(piPortDescriptors) + AMP_OUTPUT1 * SizeOf(TLADSPA_PortDescriptor))^ := LADSPA_PORT_OUTPUT or LADSPA_PORT_AUDIO;

    GetMem(pcPortNames, 3 * SizeOf(Pointer));
    g_psMonoDescriptor.PortNames := pcPortNames;
    PPChar(Integer(pcPortNames) + AMP_CONTROL * SizeOf(Pointer))^ := 'Gain';
    PPChar(Integer(pcPortNames) + AMP_INPUT1 * SizeOf(Pointer))^ := 'Input';
    PPChar(Integer(pcPortNames) + AMP_OUTPUT1 * SizeOf(Pointer))^ := 'Output';

    GetMem(psPortRangeHints, 3 * SizeOf(TLADSPA_PortRangeHint));
    g_psMonoDescriptor.PortRangeHints := psPortRangeHints;
    PLADSPA_PortRangeHint(Integer(psPortRangeHints) + AMP_CONTROL * Sizeof(TLADSPA_PortRangeHint))^.HintDescriptor := LADSPA_HINT_BOUNDED_BELOW or LADSPA_HINT_LOGARITHMIC or LADSPA_HINT_DEFAULT_1;
    PLADSPA_PortRangeHint(Integer(psPortRangeHints) + AMP_CONTROL * Sizeof(TLADSPA_PortRangeHint))^.LowerBound := 0;
    PLADSPA_PortRangeHint(Integer(psPortRangeHints) + AMP_INPUT1 * Sizeof(TLADSPA_PortRangeHint))^.HintDescriptor := 0;
    PLADSPA_PortRangeHint(Integer(psPortRangeHints) + AMP_OUTPUT1 * Sizeof(TLADSPA_PortRangeHint))^.HintDescriptor := 0;
    g_psMonoDescriptor.Instantiate := instantiateAmplifier;
    g_psMonoDescriptor.connect_port := connectPortToAmplifier;
    g_psMonoDescriptor.activate := NIL;
    g_psMonoDescriptor.run := runMonoAmplifier;
    g_psMonoDescriptor.run_adding := NIL;
    g_psMonoDescriptor.set_run_adding_gain := NIL;
    g_psMonoDescriptor.deactivate := NIL;
    g_psMonoDescriptor.cleanup := cleanupAmplifier;
  end;

  if g_psStereoDescriptor <> NIL then
  begin
    g_psStereoDescriptor.UniqueID := 1049;
    g_psStereoDescriptor.lLabel := 'amp_stereo';
    g_psStereoDescriptor.Properties := LADSPA_PROPERTY_HARD_RT_CAPABLE;
    g_psStereoDescriptor.Name := 'Stereo Amplifier';
    g_psStereoDescriptor.Maker := 'Richard Furse (LADSPA example plugins) Kylix port by Jarko Asenov';
    g_psStereoDescriptor.Copyright := 'None';
    g_psStereoDescriptor.PortCount := 5;

    GetMem(piPortDescriptors, 5 * SizeOf(TLADSPA_PortDescriptor));
    g_psStereoDescriptor.PortDescriptors := piPortDescriptors;
    PLADSPA_PortDescriptor(Integer(piPortDescriptors) + AMP_CONTROL * SizeOf(TLADSPA_PortDescriptor))^ := LADSPA_PORT_INPUT or LADSPA_PORT_CONTROL;
    PLADSPA_PortDescriptor(Integer(piPortDescriptors) + AMP_INPUT1 * SizeOf(TLADSPA_PortDescriptor))^ := LADSPA_PORT_INPUT or LADSPA_PORT_AUDIO;
    PLADSPA_PortDescriptor(Integer(piPortDescriptors) + AMP_OUTPUT1 * SizeOf(TLADSPA_PortDescriptor))^ := LADSPA_PORT_OUTPUT or LADSPA_PORT_AUDIO;
    PLADSPA_PortDescriptor(Integer(piPortDescriptors) + AMP_INPUT2 * SizeOf(TLADSPA_PortDescriptor))^ := LADSPA_PORT_INPUT or LADSPA_PORT_AUDIO;
    PLADSPA_PortDescriptor(Integer(piPortDescriptors) + AMP_OUTPUT2 * SizeOf(TLADSPA_PortDescriptor))^ := LADSPA_PORT_OUTPUT or LADSPA_PORT_AUDIO;

    GetMem(pcPortNames, 5 * SizeOf(Pointer));
    g_psStereoDescriptor.PortNames := pcPortNames;
    PPChar(Integer(pcPortNames) + AMP_CONTROL * SizeOf(Pointer))^ := 'Gain';
    PPChar(Integer(pcPortNames) + AMP_INPUT1 * SizeOf(Pointer))^ := 'Input (Left)';
    PPChar(Integer(pcPortNames) + AMP_OUTPUT1 * SizeOf(Pointer))^ := 'Output (Left)';
    PPChar(Integer(pcPortNames) + AMP_INPUT2 * SizeOf(Pointer))^ := 'Input (Right)';
    PPChar(Integer(pcPortNames) + AMP_OUTPUT2 * SizeOf(Pointer))^ := 'Output (Right)';

    GetMem(psPortRangeHints, 5 * SizeOf(TLADSPA_PortRangeHint));
    g_psStereoDescriptor.PortRangeHints := psPortRangeHints;
    PLADSPA_PortRangeHint(Integer(psPortRangeHints) + AMP_CONTROL * Sizeof(TLADSPA_PortRangeHint))^.HintDescriptor := LADSPA_HINT_BOUNDED_BELOW or LADSPA_HINT_LOGARITHMIC or LADSPA_HINT_DEFAULT_1;
    PLADSPA_PortRangeHint(Integer(psPortRangeHints) + AMP_CONTROL * Sizeof(TLADSPA_PortRangeHint))^.LowerBound := 0;
    PLADSPA_PortRangeHint(Integer(psPortRangeHints) + AMP_INPUT1 * Sizeof(TLADSPA_PortRangeHint))^.HintDescriptor := 0;
    PLADSPA_PortRangeHint(Integer(psPortRangeHints) + AMP_OUTPUT1 * Sizeof(TLADSPA_PortRangeHint))^.HintDescriptor := 0;
    PLADSPA_PortRangeHint(Integer(psPortRangeHints) + AMP_INPUT2 * Sizeof(TLADSPA_PortRangeHint))^.HintDescriptor := 0;
    PLADSPA_PortRangeHint(Integer(psPortRangeHints) + AMP_OUTPUT2 * Sizeof(TLADSPA_PortRangeHint))^.HintDescriptor := 0;
    g_psStereoDescriptor.instantiate := instantiateAmplifier;
    g_psStereoDescriptor.connect_port := connectPortToAmplifier;
    g_psStereoDescriptor.activate := NIL;
    g_psStereoDescriptor.run := runStereoAmplifier;
    g_psStereoDescriptor.run_adding := NIL;
    g_psStereoDescriptor.set_run_adding_gain := NIL;
    g_psStereoDescriptor.deactivate := NIL;
    g_psStereoDescriptor.cleanup := cleanupAmplifier;
  end;
end;

(*****************************************************************************)

procedure deleteDescriptor(psDescriptor: PLADSPA_Descriptor); cdecl;
var
  lIndex: Integer;
begin
  if psDescriptor <> NIL then
  begin
    FreeMem(psDescriptor.lLabel);
    FreeMem(psDescriptor.Name);
    FreeMem(psDescriptor.Maker);
    FreeMem(psDescriptor.Copyright);
    FreeMem(psDescriptor.PortDescriptors);
    for lIndex := 0 to psDescriptor.PortCount - 1 do
      FreeMem(PPChar(Integer(psDescriptor.PortNames) + lIndex * SizeOf(Pointer))^);
    FreeMem(psDescriptor.PortNames);
    FreeMem(psDescriptor.PortRangeHints);
    FreeMem(psDescriptor);
  end;
end;

(*****************************************************************************)

(* _fini() is called automatically when the library is unloaded. *)
procedure _fini(); cdecl;
begin
  deleteDescriptor(g_psMonoDescriptor);
  deleteDescriptor(g_psStereoDescriptor);
end;

(*****************************************************************************)

(* Return a descriptor of the requested plugin type. There are two
   plugin types available in this library (mono and stereo). *)
function ladspa_descriptor(Index: Cardinal): PLADSPA_Descriptor; cdecl;
begin
  (* Return the requested descriptor or null if the index is out of
     range. *)
  case Index of
    0: Result := g_psMonoDescriptor;
    1: Result := g_psStereoDescriptor;
  else
    Result := NIL;
  end;
end;

(*****************************************************************************)


exports
        _init name '_init',
        ladspa_descriptor name 'ladspa_descriptor',
        _fini name '_fini'
        ;

begin
  _init();

(* EOF *)
end.

